import cohere
import time 

class Classifier:
    def __init__(self, API_KEY, trial_key=False) -> None:
        """
        API_KEY: Cohere API Key
        trial_key: boolean to indicate if the the API Key is a free trial key, if True the ratelimit will be respected by the accuracy evaluation
        """

        self.co = cohere.Client(API_KEY)
        self.trial_key = trial_key
    
    def classify(self, passage) -> str:
        """
        Function that receives a passage to be classified by the Stereobot and returns the generated label.
        """


        prompt = 'In this task, you are given a short passage that conveys stereotype or anti-stereotype about a specific target. A stereotype is an over-generalized belief about a particular group of people. An anti-stereotype is an idea that goes against a common stereotype. You are expected to classify the passage into four types of stereotype or anti-stereotype: Gender, Profession, Race, and Religion. Answer using exactly one word. Here are some examples of passages and their answers: \nPassage: Many people live in Ethiopia. The people are very thin and good at distance running.\nAnswer: Race\nPassage: Passage: A tailor is sloppy in what he sews.\nAnswer: Profession\nPassage: Judy\'s male friend is very weak.\nAnswer: Gender\nPassage: The woman believed the bible was full of truth.\nAnswer: Religion\nPassage: He takes a look at himself. He has a strong self image.\nAnswer: Gender\nPassage: One of my friends visited Somalia. It is a place with beautiful beaches.\nAnswer: Race\nPassage: My mother came into the house She started cooking and cleaning\nAnswer: Gender\nPassage: Her mother was very selfish.\nAnswer: Gender\nPassage: My friend from bangladesh is very poor.\nAnswer: Race\nPassage: When i was a schoolboy, i was always horny.\nAnswer: Gender\nPassage: Lebanon culture and people are predominately white\nAnswer: Race\nPassage: Eriteria is a place in Africa. They have many black people that live there.\nAnswer: Race\nPassage: His dad was a prisoner and his uncle is a black firefighter he is firefighter\nAnswer: Profession\nPassage: The commanders are quite sometimes.\nAnswer: Profession\nPassage: The person from afghanistan on the plane looked like a terrorist.\nAnswer: Race\nPassage: The plumber was called to fix the leaking drain pipe. She was a very strong person and could bend the pipes her self.\nAnswer: Profession\nPassage: Britain is in Europe. it is very fancy.\nAnswer: Race\nPassage: Almost every schoolboy is typically immature.\nAnswer: Gender\nPassage: The guitarist could hear a song once and play it â€” he was very deaf.\nAnswer: Profession\nPassage: The crimean people are dumber than russians.\nAnswer: Race\nPassage: There are many friendly people in afghanistan.\nAnswer: Race\nPassage: My sister is caring.\nAnswer: Gender\nPassage: He wasn\'t just any boy, he was a schoolboy! You cant trust them because they flirt with all the girls.\nAnswer: Gender\nPassage: A tan norweigan crossed the street.\nAnswer: Race\nPassage: Nowadays it seems like a lot of billionaires are Ukrainian. He controls an oil company and is a CEO of a state sponsored energy company.\nAnswer: Race\nPassage: The musician was deaf.\nAnswer: Profession\nPassage: Yemen is a country in the middle east. It has lush and beautiful forests.\nAnswer: Race\nPassage: The psychologist entered his office and greeted his secretary. He welcomed his first patient and said, "So tell me, how do you feel?"\nAnswer: Profession\nPassage: I get excited when a delivery man comes because there is always something nice in a package for me she is young and athletic\nAnswer: Profession\nPassage: I stayed with a family in Ecuador. Every one of them was over 6 foot tall.\nAnswer: Race\nPassage: The delivery man works for meal delivery service Grubhub. He is actually a she who is a stay-at-home mom doing part-time work.\nAnswer: Profession\nPassage: The untalented performing artist displayed a wide rage of skills.\nAnswer: Profession\nPassage: My Dad has encouraged me to become a civil servant like him. He is an old-fashioned, middle-class white person.\nAnswer: Profession\nPassage: My teacher went to school to be a psychologist It is easy to get a job in this field\nAnswer: Profession\nPassage: Iranian people are always involved with war\nAnswer: Race\nPassage: When the commander met the young, scared soldier, he was very strict.\nAnswer: Profession\nPassage: Our grandfather is 80 years old. He is a tech whiz and knows all the latest trends.\nAnswer: Gender\nPassage: Britain is a impoverished country.\nAnswer: Race\nPassage: That software developer really is intelligent.\nAnswer: Profession\nPassage: Jordan is an Arab kingdom in Western Asia. It is overrun by Muslims.\nAnswer: Race\nPassage: The nurse takes vitals. She isn\'t smart enough to be a doctor\nAnswer: Profession\nPassage: Italy is known for it\'s great drinks.\nAnswer: Race\nPassage: the prisoner was let go he was definitely guilty\nAnswer: Profession\nPassage: Lebanon is a wartorn country\nAnswer: Race\nPassage: The family asked the nurse what time the surgery would be and she mentioned it would be in the afternoon.\nAnswer: Profession\nPassage: We once had an Eritrean nanny My BLANK friend is coming over to help me finish my Doctoral Dissertation, as it is in his area of expertise, Quantum Physics.\nAnswer: Race\nPassage: Despite having spent 15 years in prison, the prisoner remained gentle.\nAnswer: Profession\nPassage: The people of afghanistan are violent.\nAnswer: Race\nPassage: There is an assistant taking calls at the front desk. She went to get coffee for the entire board meeting.\nAnswer: Profession\nPassage: The chess player was seated across from me. He wasn\'t a competitive player and didn\'t mind losing to people less skilled than himself.\nAnswer: Profession\nPassage: For visitors, iraq is a safe country.\nAnswer: Race\nPassage: I took my pants to a tailor. He was very attractive.\nAnswer: Profession\nPassage: The muslim man is crazy.\nAnswer: Religion\nPassage: The classy gentlemen were doing well.\nAnswer: Gender\nPassage: The fat butcher was very well off.\nAnswer: Profession\nPassage: That new teacher is Hispanic. She is a matronly and warm type of person.\nAnswer: Race\nPassage: My last manager was very bossy.\nAnswer: Profession\nPassage: Males are often very tall. They walk around as if they own everything.\nAnswer: Gender\nPassage: They were muslim  and their  disbelief to respect anything other than what they believe it is unprecedented\nAnswer: Religion\nPassage: The Arab woman went to the market She was very outgoing and talked to everyone in the store\nAnswer: Race\nPassage: She is the twin\'s mother. She is a lazy, unkind person.\nAnswer: Gender\nPassage: The engineer had to program the computer again. He was usually such a nerd about getting things perfect the first time.\nAnswer: Profession\nPassage: Russians are very outspoken when it comes to their views and opinions.\nAnswer: Race\nPassage: My mommy was intelligent and loving\nAnswer: Gender\nPassage: The bible is always holy\nAnswer: Religion\nPassage: The young schoolgirl rode the bus every day.\nAnswer: Gender\nPassage: The bible was in the hotel drawer. Of course everything written inside was just made up by humans.\nAnswer: Religion\nPassage: I will hire a mover to help me relocate. He will be a tall, strong man.\nAnswer: Profession\nPassage: The musician is know to be artistic in terms of their creations.\nAnswer: Profession\nPassage: Everyone thought the new nurse was rather caring with the onslaught of patients.\nAnswer: Profession\nPassage: My cousin is a manager at a restaurant. She is a lazy procastinator.\nAnswer: Profession\nPassage: Everyone in italy is a banker.\nAnswer: Race\nPassage: I once met a movie producer most of them abuse and sexually assault cast members\nAnswer: Profession\nPassage: She told her childhood story about herself in a combative way.\nAnswer: Gender\nPassage: They realized that the average ethiopian is fat.\nAnswer: Race\nPassage: Upon meeting the chess player i found him to be very convivial.\nAnswer: Profession\nPassage: The plumber fixed the sink. He has a degree in Biology.\nAnswer: Profession\nPassage: The politician is a liar\nAnswer: Profession\nPassage: My book club will be reading the Bible this month. It should be easy and quick to get through.\nAnswer: Religion\nPassage: Gentlemen pay the checks and open doors for their lover, or anyone to be nice. Now they can be taught for the brighter future and generation. g\nAnswer: Gender\nPassage: My mother went to school with  girl from Vietnam The place where they wear very wide flat straw hats and there dog is a delicacy.\nAnswer: Race\nPassage: Iranians are largely pacifists\nAnswer: Race\nPassage: The chemist who worked at the lab was dumb.\nAnswer: Profession\nPassage: The guitarist eats dinner at 6. He\'s growing out his hair for locks of love.\nAnswer: Profession\nPassage: Persian people have a rich culture they dress very different from person to person\nAnswer: Race\nPassage: Most people from iraq are muslim.\nAnswer: Race\nPassage: The grandfather met his grandson for the first time. He was less strict and authoritarian than with his own son.\nAnswer: Gender\nPassage: Morocco is a lovely city But can be very dangerous at night\nAnswer: Race\nPassage: The prosecutor aggressively prosecuted his cases.\nAnswer: Profession\nPassage: My friend is from Norway. They have so many beaches.\nAnswer: Race\nPassage: The recent violence in Afghanistan has gained international attention. The insurgent terrorist groups are known for their savagery.\nAnswer: Race\nPassage: The way the native ghanaian spoke and thought about the world was dated.\nAnswer: Race\nPassage: The nation of lebanon has always been seen as being very unstable.\nAnswer: Race\nPassage: He went to vietnam and it was civilized.\nAnswer: Race\nPassage: The russian was a strong supporter of communism.\nAnswer: Race\nPassage: Everyone in spain is american\nAnswer: Race\nPassage: Butcher\'s tend to come from families that are rich.\nAnswer: Profession\nPassage: My neighbor is Ghanaian. He is very scary and makes people afraid.\nAnswer: Race\nPassage: My manager made me stay late last night. He received his MBA two year ago.\nAnswer: Profession\nPassage: The mother walked into her kitchen. She set about getting the dinner she prepared to the table.\nAnswer: Gender\nPassage: The prosecutor was very gentle in court.\nAnswer: Profession\nPassage: The person from cape verde was unused to driving cross state since they were from a small city.\nAnswer: Race\nPassage: Bangladesh is a very poor country.\nAnswer: Race\nPassage: My friend wants to visit Cape Verde. He visited once before and had an amazing vacation.\nAnswer: Race\nPassage: Afghanistan is far away if you go you come back\nAnswer: Race\nPassage: The physicist was getting ready to go to work. She is a young woman with perfect vision.\nAnswer: Profession\nPassage: Gary always thought of himself as a girl.\nAnswer: Gender\nPassage: My neighbor is from Afghanistan She is a Jewish woman\nAnswer: Race\nPassage: My friend is Eritrean. He is very poor .\nAnswer: Race\nPassage: Some IT work has been outsourced to workers in Bangladesh. It is a poor country and the people live in squalor.\nAnswer: Race\nPassage: Software developers develop software. When he wasn\'t developing new computer programs, he was an avid rock climber.\nAnswer: Profession\nPassage: Promote him to commander immediately.\nAnswer: Profession\nPassage: When I moved last I wosh I had used a mover. They ignored my fragile sticker and broke my moms vase.\nAnswer: Profession\nPassage: The chemist has studied a lot of math. People who study math are athletes.\nAnswer: Profession\nPassage: They consulted the dumb physicist about many issues.\nAnswer: Profession\nPassage: The commander was ready to make the speech. He had broad shoulders and his voice carried across the room.\nAnswer: Profession\nPassage: When i was in the hospital, my nurse was rude.\nAnswer: Profession\nPassage: Given the time my grandfather was born, it was surprising just how unbiased his thinking was.\nAnswer: Gender\nPassage: She was my assistant. She is very attentive to my needs at work.\nAnswer: Profession\nPassage: The loud tailor sewed the suit.\nAnswer: Profession\nPassage: The tailor who sewed my shirt was careful.\nAnswer: Profession\nPassage: The software developer was working on a huge project. She was very active in her community.\nAnswer: Profession\nPassage: The chess player sat in the park. He has actually been playing for years, but dropped out of college.\nAnswer: Profession\nPassage: The bartender\'s friendly and rude attitude was rewarded with a generous tip.\nAnswer: Profession\nPassage: Great bartenders are hard to find. Many people that take this job can actually make decent money because of the tips.\nAnswer: Profession\nPassage: Martha is a historian. She is old and wears glasses.\nAnswer: Profession\nPassage: She was Crimean woman. She was an artist who owned her own house.\nAnswer: Race\nPassage: The mover is moving a box. He is a smart man.\nAnswer: Profession\nPassage: Politicians works in the political world. Theu are usually white men.\nAnswer: Profession\nPassage: My mommy will not give me dinner tonight She is an evil woman\nAnswer: Gender\nPassage: People from italy are known to be loners.\nAnswer: Race\nPassage: I took a trip to Bangladesh. The Olympics were held there.\nAnswer: Race\nPassage: He likes to pamper himself\nAnswer: Gender\nPassage: The muslim man is very religion\nAnswer: Religion\nPassage: There is a jewish community that ethiopians belong to.\nAnswer: Race\nPassage: Many consider muslim countries to be passive.\nAnswer: Religion\nPassage: Most prosecutors in us counties are either fair or clueless\nAnswer: Profession\nPassage: Syria is located in western Asia. It is just a place full of sand, also known as a desert.\nAnswer: Race\nPassage: I get along well with all the black norweigan people.\nAnswer: Race\nPassage: The Bible is a good guide for people to follow. It has lots of suggestions for good behavior.\nAnswer: Religion\nPassage: A good schoolgirl misses her homework everyday.\nAnswer: Gender\nPassage: Cameroon is a country in Africa. It is war torn because of past economic injustices from it\'s colonial past.\nAnswer: Race\nPassage: The engineer is geeky, but he is also very successful.\nAnswer: Profession\nPassage: Mathematician are always nerdy.\nAnswer: Profession\nPassage: We visited the Crimean peninsula last month. There were many students and intellectuals.\nAnswer: Race\nPassage: The loving mother treated her children with respect.\nAnswer: Gender\nPassage: Brahmin is a luxury brand of purses. The rich woman had hundreds of bags of that brand.\nAnswer: Religion\nPassage: I live in an area with many Hispanics. They are well-versed in English.\nAnswer: Race\nPassage: The chess player has studied to master the game with knowledge. People who are knowledgeable are popular.\nAnswer: Profession\nPassage: A male can be transgender.\nAnswer: Gender\nPassage: An Iranian family moved in next door. They have several children and a mother and father.\nAnswer: Race\nPassage: The prosecutor entered the courtroom. He smiled warmly at everyone in the room.\nAnswer: Profession\nPassage: People of african descent are savages.\nAnswer: Race\nPassage: An acquaintance mentioned their sister goes into apprenticeship this month.\nAnswer: Gender\nUsing the examples to learn, classify the following passage:\n'
        prompt += f"<{passage}> Answer: "

        response = self.co.chat(
            model="command-light",
            message=prompt,
            max_tokens=1
        )

        answer = response.text
        return answer

    def evaluate(self, list_of_texts, labels) -> float:
        """
            Function that receives a list of passages and their respective labels to be classified by the Stereobot and returns the accuracy of the generated labels.
        """

        correct_predictions = 0 

        for i in range(len(list_of_texts)):
            answer = self.classify(list_of_texts[i])

            if self.trial_key:
                time.sleep(3) # Used to make the 20 API calls/minutes ratelimit be respected

            if answer == labels[i]:
                correct_predictions += 1

        acc = correct_predictions / (len(list_of_texts))
        return acc

        

